@@ -1,4 +1,4 @@
-# Copyright 2001-2005, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,13 +10,13 @@ package Devel::Cover;
 use strict;
 use warnings;
 
-our $VERSION = "0.55";
+our $VERSION = "0.59";
 
 use DynaLoader ();
 our @ISA = "DynaLoader";
 
-use Devel::Cover::DB  0.55;
-use Devel::Cover::Inc 0.55;
+use Devel::Cover::DB  0.59;
+use Devel::Cover::Inc 0.59;
 
 use B qw( class ppname main_cv main_start main_root walksymtable OPf_KIDS );
 use B::Debug;
@@ -31,8 +31,12 @@ BEGIN
 {
     # Use Pod::Coverage if it is available.
     eval "use Pod::Coverage 0.06";
+    # If there is any error other than a failure to locate, report it.
+    die $@ if $@ && $@ !~ m/Can't locate Pod\/Coverage.+pm in \@INC/;
+
     # We'll prefer Pod::Coverage::CountParents
     eval "use Pod::Coverage::CountParents";
+    die $@ if $@ && $@ !~ m/Can't locate Pod\/Coverage.+pm in \@INC/;
 }
 
 # $SIG{__DIE__} = \&Carp::confess;
@@ -41,9 +45,10 @@ my $Initialised;                         # import() has been called.
 
 my $Dir;                                 # Directory in which coverage will be
                                          # collected.
-my $DB      = "cover_db";                # DB name.
-my $Merge   = 1;                         # Merge databases.
-my $Summary = 1;                         # Output coverage summary.
+my $DB        = "cover_db";              # DB name.
+my $Merge     = 1;                       # Merge databases.
+my $Summary   = 1;                       # Output coverage summary.
+my $Subs_only = 0;                       # Coverage only for sub bodies.
 
 my @Ignore;                              # Packages to ignore.
 my @Inc;                                 # Original @INC to ignore.
@@ -58,6 +63,7 @@ my $Pod = $INC{"Pod/Coverage/CountParents.pm"} ? "Pod::Coverage::CountParents" :
 my %Pod;                                 # Pod coverage data.
 
 my @Cvs;                                 # All the Cvs we want to cover.
+my @Subs;                                # All the subs we want to cover.
 my $Cv;                                  # Cv we are looking in.
 my $Sub_name;                            # Name of the sub we are looking in.
 
@@ -172,14 +178,15 @@ EOM
 
         set_coverage(keys %Coverage);
         @coverage = get_coverage();
-        my $last = pop @coverage;
+        my $last = pop @coverage || "";
 
         print STDOUT __PACKAGE__, " $VERSION: Collecting coverage data for ",
               join(", ", @coverage),
               @coverage ? " and " : "",
               "$last.\n",
               $nopod,
-              $ENV{MOD_PERL} ? "    Collecting under $ENV{MOD_PERL}\n" : "",
+              $Subs_only     ? "    Collecting for subroutines only.\n" : "",
+              $ENV{MOD_PERL} ? "    Collecting under $ENV{MOD_PERL}\n"  : "",
               "Selecting packages matching:", join("\n    ", "", @Select), "\n",
               "Ignoring packages matching:",  join("\n    ", "", @Ignore), "\n",
               "Ignoring packages in:",        join("\n    ", "", @Inc),    "\n"
@@ -220,8 +227,8 @@ sub last_end
 
 {
     no warnings "void";  # avoid "Too late to run ... block" warning
-    INIT  {}  # dummy sub to make sure PL_endav  is set up and populated
-    END   {}  # dummy sub to make sure PL_initav is set up and populated
+    INIT  {}  # dummy sub to make sure PL_initav is set up and populated
+    END   {}  # dummy sub to make sure PL_endav  is set up and populated
     CHECK { set_first_init_and_end() }  # we really want to be first
 }
 
@@ -255,12 +262,13 @@ sub import
     while (@o)
     {
         local $_ = shift @o;
-        /^-silent/    && do { $Silent  = shift @o; next };
-        /^-dir/       && do { $Dir     = shift @o; next };
-        /^-db/        && do { $DB      = shift @o; next };
-        /^-merge/     && do { $Merge   = shift @o; next };
-        /^-summary/   && do { $Summary = shift @o; next };
-        /^-blib/      && do { $blib    = shift @o; next };
+        /^-silent/    && do { $Silent    = shift @o; next };
+        /^-dir/       && do { $Dir       = shift @o; next };
+        /^-db/        && do { $DB        = shift @o; next };
+        /^-merge/     && do { $Merge     = shift @o; next };
+        /^-summary/   && do { $Summary   = shift @o; next };
+        /^-blib/      && do { $blib      = shift @o; next };
+        /^-subs_only/ && do { $Subs_only = shift @o; next };
         /^-coverage/  &&
             do { $Coverage{+shift @o} = 1 while @o && $o[0] !~ /^[-+]/; next };
         /^[-+]ignore/ &&
@@ -288,8 +296,8 @@ sub import
     unless (-d $DB)
     {
         # Nasty hack to keep 5.6.1 happy.
-        mkdir $DB, 0777 or croak "Can't mkdir $DB: $!\n";
-        chmod 0777, $DB or croak "Can't chmod $DB: $!\n";
+        mkdir $DB, 0700 or croak "Can't mkdir $DB: $!\n";
+        chmod 0700, $DB or croak "Can't chmod $DB: $!\n";
     }
     $DB = $1 if Cwd::abs_path($DB) =~ /(.*)/;
     Devel::Cover::DB->delete($DB) unless $Merge;
@@ -522,9 +530,10 @@ sub sub_info
     {
         return unless $cv->GV->can("SAFENAME");
         $name = $cv->GV->SAFENAME;
+        # print "--[$name]--\n";
         $name =~ s/(__ANON__)\[.+:\d+\]/$1/ if defined $name;
     }
-    my $root  = $cv->ROOT;
+    my $root = $cv->ROOT;
     if ($root->can("first"))
     {
         my $lineseq = $root->first;
@@ -563,6 +572,7 @@ sub check_files
             local ($Line, $File);
             get_location($start);
             $line = $Line;
+            # print "$name - $File:$Line\n";
         }
         ($line, $name)
     };
@@ -572,6 +582,13 @@ sub check_files
            map  [ $_, $l->($_) ],
            grep !$seen_cv{$$_}++,
            @Cvs;
+
+    # Hack to bump up the refcount of the subs.  If we don't do this then the
+    # subs in some modules don't seem to be around when we get to looking at
+    # them.  I'm not sure why this is, and it seems to me that this hack could
+    # affect the order of destruction, but I've not seen any problems.  Yet.
+    # object_2svref doesn't exist before 5.8.1.
+    @Subs = map $_->object_2svref, @Cvs if $] >= 5.008001;
 }
 
 sub report
@@ -600,14 +617,21 @@ sub report
 
     check_files();
 
-    get_cover(main_cv, main_root);
-    get_cover($_) for B::begin_av()->isa("B::AV") ? B::begin_av()->ARRAY : ();
-    if (exists &B::check_av)
+    unless ($Subs_only)
     {
+        get_cover(main_cv, main_root);
         get_cover($_)
-            for B::check_av()->isa("B::AV") ? B::check_av()->ARRAY : ();
+            for B::begin_av()->isa("B::AV") ? B::begin_av()->ARRAY : ();
+        if (exists &B::check_av)
+        {
+            get_cover($_)
+                for B::check_av()->isa("B::AV") ? B::check_av()->ARRAY : ();
+        }
+        # get_ends includes INIT blocks
+        get_cover($_)
+            for get_ends()->isa("B::AV") ? get_ends()->ARRAY : ();
     }
-    get_cover($_) for get_ends()->isa("B::AV") ? get_ends()->ARRAY : ();
+    # print "--- @Cvs\n";
     get_cover($_) for @Cvs;
 
     my %files;
@@ -647,8 +671,8 @@ sub report
     $DB .= "/runs";
     unless (-d $DB)
     {
-        mkdir $DB, 0777 or croak "Can't mkdir $DB: $!\n";
-        chmod 0777, $DB or croak "Can't chmod $DB: $!\n";
+        mkdir $DB, 0700 or croak "Can't mkdir $DB: $!\n";
+        chmod 0700, $DB or croak "Can't chmod $DB: $!\n";
     }
     $DB .= "/$run";
 
@@ -672,7 +696,7 @@ sub add_subroutine_cover
     my $key = get_key($op);
     my $val = $Coverage->{statement}{$key} || 0;
     my ($n, $new) = $Structure->add_count("subroutine");
-    # print STDERR "******* subroutine $n\n";
+    # print STDERR "******* subroutine $n - $new\n";
     $Structure->add_subroutine($File, [ $Line, $Sub_name ]) if $new;
     $Run{count}{$File}{subroutine}[$n] += $val;
     my $vec = $Run{vec}{$File}{subroutine};
@@ -789,7 +813,7 @@ sub add_condition_cover
         $name = $r->first->name if $name eq "sassign";
         # TODO - exec?  any others?
         # print STDERR "Name [$name]\n";
-        if ($name =~ /^const|s?refgen|gelem|die|undef|bless$/)
+        if ($c->[5] || $name =~ /^const|s?refgen|gelem|die|undef|bless$/)
         {
             $c = [ $c->[3], $c->[1] + $c->[2] ];
             $count = 2;
@@ -1101,6 +1125,13 @@ sub get_cover
         }
     }
 
+    # my $dd = @_ && ref $_[0]
+                 # ? $deparse->deparse($_[0], 0)
+                 # : $deparse->deparse_sub($cv, 0);
+    # print "get_cover: <$Sub_name>\n";
+    # print "[[$File:$Line]]\n";
+    # print "<$dd>\n";
+
     no warnings "redefine";
     local *B::Deparse::deparse     = \&deparse;
     local *B::Deparse::logop       = \&logop;
@@ -1109,7 +1140,6 @@ sub get_cover
     my $de = @_ && ref $_[0]
                  ? $deparse->deparse($_[0], 0)
                  : $deparse->deparse_sub($cv, 0);
-
     # print "<$de>\n";
     $de
 }
@@ -1137,13 +1167,21 @@ To test an uninstalled module:
  HARNESS_PERL_SWITCHES=-MDevel::Cover make test
  cover
 
+To test an uninstalled module which uses Module::Build (0.26 or later):
+
+ ./Build testcover
+
 If the module does not use the t/*.t framework:
 
  PERL5OPT=-MDevel::Cover make test
 
 =head1 DESCRIPTION
 
-This module provides code coverage metrics for Perl.
+This module provides code coverage metrics for Perl. Code coverage
+metrics describe how throroughly tests exercise code. By using
+Devel::Cover you can find areas of code not exercised by your tests
+and find out which tests to create to increase coverage. Code coverage
+can be considered as an indirect measure of quality.
 
 If you can't guess by the version number this is an alpha release.
 
@@ -1194,6 +1232,10 @@ This means a working compiler and make program at least.
 
 Both are in the core in Perl 5.8.0 and above.
 
+=item * L<Template> and L<PPI::HTML> or L<Perl::Tidy>
+
+if you want syntax highlighted HTML reports.
+
 =item * L<Pod::Coverage>
 
 if you want Pod coverage.
@@ -1222,6 +1264,7 @@ if the tests fail and you would like nice output telling you why.
  -select RE          - Set REs of files to select (default none).
  +select RE          - Append to REs of files to select.
  -silent val         - Don't print informational messages (default off)
+ -subs_only val      - Only cover code in subroutine bodies (default off)
  -summary val        - Print summary information iff val is true (default on).
 
 =head2 More on Coverage Options
@@ -1250,10 +1293,13 @@ You may add to the REs to select by using +select, or you may reset the
 selections using -select.  The same principle applies to the REs to
 ignore.
 
-The inc directories are initially populated with the contects of the
+The inc directories are initially populated with the contents of the
 @INC array at the time Devel::Cover was built.  You may reset these
 directories using -inc, or add to them using +inc.
 
+Although these options take regular expressions, you should not enclose the RE
+within //.
+
 =head1 ENVIRONMENT
 
 The -silent option is turned on when Devel::Cover is invoked via
@@ -1342,6 +1388,12 @@ mod_perl.  You can also add any options you need at this point.  I would
 suggest adding this as early as possible in your startup script in order
 to collect as much coverage information as possible.
 
+=head2 Redefined subroutines
+
+If you redefine a subroutine you may find that the original subroutine is not
+reported on.  This is because I haven't yet found a way to locate the original
+CV.  Hints, tips or patches to resolve this will be gladly accepted.
+
 =head1 BUGS
 
 Did I mention that this is alpha code?
@@ -1350,11 +1402,11 @@ See the BUGS file.  And the TODO file.
 
 =head1 VERSION
 
-Version 0.55 - 22nd September 2005
+Version 0.59 - 23rd August 2006
 
 =head1 LICENCE
 
-Copyright 2001-2005, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
