@@ -1,4 +1,4 @@
-# Copyright 2001-2005, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,7 +10,7 @@ package Devel::Cover::Report::Html;
 use strict;
 use warnings;
 
-our $VERSION = "0.55";
+our $VERSION = "0.59";
 
 use base "Devel::Cover::Report::Html_minimal";
 
@@ -33,7 +33,8 @@ statistics
 
 This module provides a HTML reporting mechanism for coverage data.  It
 is designed to be called from the C<cover> program.  This is an empty
-class derived from the default HTML output module.
+class derived from the default HTML output module,
+Devel::Cover::Report::Html_minimal.
 
 =head1 SEE ALSO
 
@@ -45,11 +46,11 @@ Huh?
 
 =head1 VERSION
 
-Version 0.55 - 22nd September 2005
+Version 0.59 - 23rd August 2006
 
 =head1 LICENCE
 
-Copyright 2001-2005, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
