@@ -1,4 +1,4 @@
-# Copyright 2002-2005, Paul Johnson (pjcj@cpan.org)
+# Copyright 2002-2006, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,14 +10,14 @@ package Devel::Cover::Test;
 use strict;
 use warnings;
 
-our $VERSION = "0.55";
+our $VERSION = "0.59";
 
 use Carp;
 
 use File::Spec;
 use Test;
 
-use Devel::Cover::Inc 0.55;
+use Devel::Cover::Inc 0.59;
 
 my $Test;
 
@@ -67,7 +67,8 @@ sub get_params
     $self->{select}         ||= "-select $self->{test}";
     $self->{test_parameters}  = "$self->{select}"
                               . " -ignore blib Devel/Cover $self->{ignore}"
-                              . " -merge 0 -coverage $self->{criteria}";
+                              . " -merge 0 -coverage $self->{criteria} "
+                              . ($self->{test_parameters} || "");
     $self->{criteria} =~ s/-\w+//g;
     $self->{cover_parameters} = join(" ", map "-coverage $_",
                                               split " ", $self->{criteria})
